package hol;

import java.util.ArrayList;
import java.util.List;

import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.JobConf;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.api.java.JavaSparkContext;
import oracle.spatial.geometry.JGeometry;
import oracle.spatial.hadoop.vector.util.JobUtils;
import oracle.spatial.spark.vector.SparkRecordInfo;
import oracle.spatial.spark.vector.rdd.SpatialJavaRDD;
import oracle.spatial.spark.vector.recordinfoprovider.GeoJsonRecordInfoProvider;
import oracle.spatial.spark.vector.serialization.SpatialVectorKryoRegistrator;
import scala.Tuple2;
import oracle.spatial.hadoop.vector.geojson.mapred.GeoJsonInputFormat;
import oracle.spatial.hadoop.vector.util.SpatialOperation;
import oracle.spatial.hadoop.vector.util.SpatialOperationConfig;
import oracle.spatial.spark.vector.SparkRecordInfoProvider;

public class NearestNeighbors {
    public NearestNeighbors() {
        super();
    }

    public static void main(String[] args) throws Exception {
        //Create a JobConf
        JobConf jobConf = new JobConf();
        //set input path
        JobUtils.setupInputPath(args[0], jobConf);

        // Create a Java Spark Context
        SparkConf conf = new SparkConf().setAppName("NearestNeighbors");

        //use the Kryo Serializer
        conf.set("spark.serializer", "org.apache.spark.serializer.KryoSerializer");
        //set SpatialVectorKryoRegistrator as Kryo Registrator
        SpatialVectorKryoRegistrator.register(conf);

        JavaSparkContext sparkContext = new JavaSparkContext(conf);

        //load text file RDD
        JavaPairRDD<LongWritable, Text> textRDD =
            sparkContext.hadoopRDD(jobConf, GeoJsonInputFormat.class, LongWritable.class, Text.class);

        //list of GeoJSON properties to add to the record extra fields
        List<String> properties = new ArrayList<String>();
        properties.add("friends_count");
        properties.add("location");
        properties.add("followers_count");
        properties.add("user_id");

        //create RecordInfoProvider
        GeoJsonRecordInfoProvider recordInfoProvider = new GeoJsonRecordInfoProvider(properties, 2, 8307);

        //create a spatial RDD
        SpatialJavaRDD<SparkRecordInfo> spatialRDD = SpatialJavaRDD.fromJavaRDD(textRDD.values(), recordInfoProvider);

        //create a point having the coordinates of Paris
        JGeometry centerParis = JGeometry.createPoint(new double[] { 2.349014, 48.864716 }, //coordinates of Paris
                                                      2, //dimensions
                                                      8307 //SRID
                                                      );

        //get the returned record info provider
        SparkRecordInfoProvider<SparkRecordInfo> rip = spatialRDD.getRecordInfoProvider();

        //determine the 10 nearest tweets of Paris
        //Note than the first element of the returned tuple is a double representing the distance
        //between the center of Paris and the tweet in meters
        List<Tuple2<Double, SparkRecordInfo>> nearestNeighbors = spatialRDD.nearestNeighbors(value -> {
                SparkRecordInfo recordInfo = new SparkRecordInfo();
                //fill recordInfo with the current record value
                if (rip.getRecordInfo(value, recordInfo)) {
                    //return only tweets with a followers_count attribute greater than 50
                    if(Long.parseLong(recordInfo.getField("followers_count").toString()) > 50){
                        return true;
                    }

                }
                
                return false;
            }, 
            10, //number of neighbors 
            new SpatialOperationConfig(SpatialOperation.None, 
                                       centerParis, 
                                       0.05 //tolerance
                                        )
        );
        
        //print the returned tweets
        nearestNeighbors.forEach(nearestNeighbor -> System.out.println("Tweet: " + nearestNeighbor.toString()));

        //close context
        sparkContext.close();
    }
}
